# -*- coding: utf-8 -*-
__version__ = '1.4.1'
__license__ = 'GPLv3'

__author__ = 'José Antonio Perdiguero López'
__email__ = 'perdy.hh@gmail.com'

__url__ = 'https://github.com/PeRDy/Clinner'
__description__ = 'command Line Interface builder that helps creating an entry point for your application'
